@@ -17,19 +17,22 @@ use Locale::Maketext::Simple    Style => 'gettext';
 use constant ON_SOLARIS     => $^O eq 'solaris' ? 1 : 0;
 use constant FILE_EXISTS    => sub { -e $_[0] ? 1 : 0 };
 
+### If these are changed, update @TYPES and the new() POD
 use constant TGZ            => 'tgz';
 use constant TAR            => 'tar';
 use constant GZ             => 'gz';
 use constant ZIP            => 'zip';
 use constant BZ2            => 'bz2';
 use constant TBZ            => 'tbz';
+use constant Z              => 'Z';
 
 use vars qw[$VERSION $PREFER_BIN $PROGRAMS $WARN $DEBUG];
 
-$VERSION        = '0.12';
+$VERSION        = '0.22';
 $PREFER_BIN     = 0;
 $WARN           = 1;
 $DEBUG          = 0;
+my @Types       = ( TGZ, TAR, GZ, ZIP, BZ2, TBZ, Z ); # same as all constants
 
 local $Params::Check::VERBOSE = $Params::Check::VERBOSE = 1;
 
@@ -84,9 +87,9 @@ Archive::Extract - A generic archive extracting mechanism
 Archive::Extract is a generic archive extraction mechanism.
 
 It allows you to extract any archive file of the type .tar, .tar.gz,
-.gz, tar.bz, .tbz2, .bz2 or .zip without having to worry how it does 
-so, or use different interfaces for each type by using either perl 
-modules, or commandline tools on your system.
+.gz, .Z, tar.bz2, .tbz, .bz2 or .zip without having to worry how it 
+does so, or use different interfaces for each type by using either 
+perl modules, or commandline tools on your system.
 
 See the C<HOW IT WORKS> section further down for details.
 
@@ -95,7 +98,7 @@ See the C<HOW IT WORKS> section further down for details.
 
 ### see what /bin/programs are available ###
 $PROGRAMS = {};
-for my $pgm (qw[tar unzip gzip bunzip2]) {
+for my $pgm (qw[tar unzip gzip bunzip2 uncompress]) {
     $PROGRAMS->{$pgm} = can_run($pgm);
 }
 
@@ -107,17 +110,19 @@ my $Mapping = {
     is_zip  => '_unzip',
     is_tbz  => '_untar',
     is_bz2  => '_bunzip2',
+    is_Z    => '_uncompress',
 };
 
 {
     my $tmpl = {
         archive => { required => 1, allow => FILE_EXISTS },
-        type    => { default => '', allow => [qw|tgz tar zip gz bz2 tbz2|] },
+        type    => { default => '', allow => [ @Types ] },
     };
 
     ### build accesssors ###
-    for my $method( keys %$tmpl,
-                    qw[_extractor _gunzip_to files extract_path]
+    for my $method( keys %$tmpl, 
+                    qw[_extractor _gunzip_to files extract_path],
+                    qw[_error_msg _error_msg_long]
     ) {
         no strict 'refs';
         *$method = sub {
@@ -155,6 +160,11 @@ Corresponds to a C<.tgz> or C<.tar.gz> suffix.
 Gzip compressed file, as produced by, for example C</bin/gzip>.
 Corresponds to a C<.gz> suffix.
 
+=item Z
+
+Lempel-Ziv compressed file, as produced by, for example C</bin/compress>.
+Corresponds to a C<.Z> suffix.
+
 =item zip
 
 Zip compressed file, as produced by, for example C</bin/zip>.
@@ -189,12 +199,13 @@ Returns a C<Archive::Extract> object on success, or false on failure.
         ### figure out the type, if it wasn't already specified ###
         unless ( $parsed->{type} ) {
             $parsed->{type} =
-                $ar =~ /.+?\.(?:tar\.gz)|tgz$/i     ? TGZ   :
+                $ar =~ /.+?\.(?:tar\.gz|tgz)$/i     ? TGZ   :
                 $ar =~ /.+?\.gz$/i                  ? GZ    :
                 $ar =~ /.+?\.tar$/i                 ? TAR   :
                 $ar =~ /.+?\.(zip|jar|par)$/i       ? ZIP   :
                 $ar =~ /.+?\.(?:tbz|tar\.bz2?)$/i   ? TBZ   :
                 $ar =~ /.+?\.bz2$/i                 ? BZ2   :
+                $ar =~ /.+?\.Z$/                    ? Z     :
                 '';
 
         }
@@ -269,9 +280,9 @@ sub extract {
     ### to.
     my $dir;
     {   ### a foo.gz file
-        if( $self->is_gz or $self->is_bz2 ) {
+        if( $self->is_gz or $self->is_bz2 or $self->is_Z) {
     
-            my $cp = $self->archive; $cp =~ s/\.(?:gz|bz2)$//i;
+            my $cp = $self->archive; $cp =~ s/\.(?:gz|bz2|Z)$//i;
         
             ### to is a dir?
             if ( -d $to ) {
@@ -370,6 +381,15 @@ This is the type of archive represented by this C<Archive::Extract>
 object. See accessors below for an easier way to use this.
 See the C<new()> method for details.
 
+=head2 $ae->types
+
+Returns a list of all known C<types> for C<Archive::Extract>'s
+C<new> method.
+
+=cut
+
+sub types { return @Types }
+
 =head2 $ae->is_tgz
 
 Returns true if the file is of type C<.tar.gz>.
@@ -385,6 +405,11 @@ See the C<new()> method for details.
 Returns true if the file is of type C<.gz>.
 See the C<new()> method for details.
 
+=head2 $ae->is_Z
+
+Returns true if the file is of type C<.Z>.
+See the C<new()> method for details.
+
 =head2 $ae->is_zip
 
 Returns true if the file is of type C<.zip>.
@@ -399,6 +424,7 @@ sub is_gz   { return $_[0]->type eq GZ  }
 sub is_zip  { return $_[0]->type eq ZIP }
 sub is_tbz  { return $_[0]->type eq TBZ }
 sub is_bz2  { return $_[0]->type eq BZ2 }
+sub is_Z    { return $_[0]->type eq Z   }
 
 =pod
 
@@ -417,10 +443,12 @@ Returns the full path to your unzip binary, if found
 =cut
 
 ### paths to commandline tools ###
-sub bin_gzip    { return $PROGRAMS->{'gzip'}    if $PROGRAMS->{'gzip'}  }
-sub bin_unzip   { return $PROGRAMS->{'unzip'}   if $PROGRAMS->{'unzip'} }
-sub bin_tar     { return $PROGRAMS->{'tar'}     if $PROGRAMS->{'tar'}   }
-sub bin_bunzip2 { return $PROGRAMS->{'bunzip2'} if $PROGRAMS->{'bunzip2'} }
+sub bin_gzip        { return $PROGRAMS->{'gzip'}    if $PROGRAMS->{'gzip'}  }
+sub bin_unzip       { return $PROGRAMS->{'unzip'}   if $PROGRAMS->{'unzip'} }
+sub bin_tar         { return $PROGRAMS->{'tar'}     if $PROGRAMS->{'tar'}   }
+sub bin_bunzip2     { return $PROGRAMS->{'bunzip2'} if $PROGRAMS->{'bunzip2'} }
+sub bin_uncompress  { return $PROGRAMS->{'uncompress'} 
+                                                 if $PROGRAMS->{'uncompress'} }
 
 #################################
 #
@@ -434,10 +462,9 @@ sub bin_bunzip2 { return $PROGRAMS->{'bunzip2'} if $PROGRAMS->{'bunzip2'} }
 sub _untar {
     my $self = shift;
 
-    ### no bzip2 support in A::T yet
-    my   @methods = qw[_untar_bin];
-    push @methods,  qw[_untar_at] unless $self->is_tbz;
-         @methods = reverse @methods unless $PREFER_BIN;
+    ### bzip2 support in A::T via IO::Uncompress::Bzip2
+    my   @methods = qw[_untar_at _untar_bin];
+         @methods = reverse @methods if $PREFER_BIN;
 
     for my $method (@methods) {
         $self->_extractor($method) && return 1 if $self->$method();
@@ -491,10 +518,10 @@ sub _untar_bin {
                             $self->archive, $buffer ));
         }
 
-        unless( $buffer ) {
-            $self->_error(loc("No buffer captured, unable to tell ".
-                              "extracted files or extraction dir for '%1'",
-                              $self->archive));
+        ### no buffers available?
+        if( !IPC::Cmd->can_capture_buffer and !$buffer ) {
+            $self->_error( $self->_no_buffer_files( $self->archive ) );
+        
         } else {
             ### if we're on solaris we /might/ be using /bin/tar, which has
             ### a weird output format... we might also be using
@@ -561,6 +588,9 @@ sub _untar_at {
         }
     }
 
+    ### we might pass it a filehandle if it's a .tbz file..
+    my $fh_to_read = $self->archive;
+
     ### we will need Compress::Zlib too, if it's a tgz... and IO::Zlib
     ### if A::T's version is 0.99 or higher
     if( $self->is_tgz ) {
@@ -576,11 +606,28 @@ sub _untar_at {
                                 "install it as soon as possible.", $which));
 
         }
+    } elsif ( $self->is_tbz ) {
+        my $use_list = { 'IO::Uncompress::Bunzip2' => '0.0' };
+        unless( can_load( modules => $use_list ) ) {
+            return $self->_error(loc(
+                    "You do not have '%1' installed - Please " .
+                    "install it as soon as possible.", 
+                     'IO::Uncompress::Bunzip2'));
+        }
+
+        my $bz = IO::Uncompress::Bunzip2->new( $self->archive ) or
+            return $self->_error(loc("Unable to open '%1': %2",
+                            $self->archive,
+                            $IO::Uncompress::Bunzip2::Bunzip2Error));
+
+        $fh_to_read = $bz;
     }
 
     my $tar = Archive::Tar->new();
 
-    unless( $tar->read( $self->archive, $self->is_tgz ) ) {
+    ### only tell it it's compressed if it's a .tgz, as we give it a file
+    ### handle if it's a .tbz
+    unless( $tar->read( $fh_to_read, ( $self->is_tgz ? 1 : 0 ) ) ) {
         return $self->_error(loc("Unable to read '%1': %2", $self->archive,
                                     $Archive::Tar::error));
     }
@@ -668,9 +715,9 @@ sub _gunzip_bin {
                                     $self->archive, $buffer));
     }
 
-    unless( $buffer ) {
-        $self->_error(loc("No buffer captured, unable to get content for '%1'",
-                          $self->archive));
+    ### no buffers available?
+    if( !IPC::Cmd->can_capture_buffer and !$buffer ) {
+        $self->_error( $self->_no_buffer_content( $self->archive ) );
     }
 
     print $fh $buffer if defined $buffer;
@@ -712,6 +759,68 @@ sub _gunzip_cz {
     return 1;
 }
 
+#################################
+#
+# Uncompress code
+#
+#################################
+
+
+### untar wrapper... goes to either Archive::Tar or /bin/tar
+### depending on $PREFER_BIN
+sub _uncompress {
+    my $self = shift;
+
+    my   @methods = qw[_gunzip_cz _uncompress_bin];
+         @methods = reverse @methods if $PREFER_BIN;
+
+    for my $method (@methods) {
+        $self->_extractor($method) && return 1 if $self->$method();
+    }
+
+    return $self->_error(loc("Unable to untar file '%1'", $self->archive));
+}
+
+sub _uncompress_bin {
+    my $self = shift;
+
+    ### check for /bin/gzip -- we need it ###
+    return $self->_error(loc("No '%1' program found", '/bin/uncompress'))
+        unless $self->bin_uncompress;
+
+
+    my $fh = FileHandle->new('>'. $self->_gunzip_to) or
+        return $self->_error(loc("Could not open '%1' for writing: %2",
+                            $self->_gunzip_to, $! ));
+
+    my $cmd = [ $self->bin_uncompress, '-c', $self->archive ];
+
+    my $buffer;
+    unless( scalar run( command => $cmd,
+                        verbose => $DEBUG,
+                        buffer  => \$buffer )
+    ) {
+        return $self->_error(loc("Unable to uncompress '%1': %2",
+                                    $self->archive, $buffer));
+    }
+
+    ### no buffers available?
+    if( !IPC::Cmd->can_capture_buffer and !$buffer ) {
+        $self->_error( $self->_no_buffer_content( $self->archive ) );
+    }
+
+    print $fh $buffer if defined $buffer;
+
+    close $fh;
+
+    ### set what files where extract, and where they went ###
+    $self->files( [$self->_gunzip_to] );
+    $self->extract_path( File::Spec->rel2abs(cwd()) );
+
+    return 1;
+}
+
+
 #################################
 #
 # Unzip code
@@ -753,10 +862,9 @@ sub _unzip_bin {
                                         $self->archive, $buffer));
         }
 
-        unless( $buffer ) {
-            $self->_error(loc("No buffer captured, unable to tell extracted ".
-                              "files or extraction dir for '%1'",
-                              $self->archive));
+        ### no buffers available?
+        if( !IPC::Cmd->can_capture_buffer and !$buffer ) {
+            $self->_error( $self->_no_buffer_files( $self->archive ) );
 
         } else {
             $self->files( [split $/, $buffer] );
@@ -824,7 +932,7 @@ sub _unzip_az {
 sub __get_extract_dir {
     my $self    = shift;
     my $files   = shift || [];
-    
+
     return unless scalar @$files;
 
     my($dir1, $dir2);
@@ -833,9 +941,11 @@ sub __get_extract_dir {
 
         ### add a catdir(), so that any trailing slashes get
         ### take care of (removed)
+        ### also, a catdir() normalises './dir/foo' to 'dir/foo';
+        ### which was the problem in bug #23999
         my $res = -d $files->[$pos]
                     ? File::Spec->catdir( $files->[$pos], '' )
-                    : dirname( $files->[$pos] );
+                    : File::Spec->catdir( dirname( $files->[$pos] ) ); 
 
         $$dir = $res;
     }
@@ -870,8 +980,8 @@ sub __get_extract_dir {
 sub _bunzip2 {
     my $self = shift;
 
-    my @methods = qw[_bunzip2_bin];
-    #   @methods = reverse @methods if $PREFER_BIN;
+    my @methods = qw[_bunzip2_cz2 _bunzip2_bin];
+       @methods = reverse @methods if $PREFER_BIN;
 
     for my $method (@methods) {
         $self->_extractor($method) && return 1 if $self->$method();
@@ -903,11 +1013,11 @@ sub _bunzip2_bin {
                                     $self->archive, $buffer));
     }
 
-    unless( $buffer ) {
-        $self->_error(loc("No buffer captured, unable to get content for '%1'",
-                          $self->archive));
+    ### no buffers available?
+    if( !IPC::Cmd->can_capture_buffer and !$buffer ) {
+        $self->_error( $self->_no_buffer_content( $self->archive ) );
     }
-
+    
     print $fh $buffer if defined $buffer;
 
     close $fh;
@@ -919,6 +1029,59 @@ sub _bunzip2_bin {
     return 1;
 }
 
+### using cz2, the compact versions... this we use mainly in archive::tar
+### extractor..
+# sub _bunzip2_cz1 {
+#     my $self = shift;
+# 
+#     my $use_list = { 'IO::Uncompress::Bunzip2' => '0.0' };
+#     unless( can_load( modules => $use_list ) ) {
+#         return $self->_error(loc("You do not have '%1' installed - Please " .
+#                         "install it as soon as possible.",
+#                         'IO::Uncompress::Bunzip2'));
+#     }
+# 
+#     my $bz = IO::Uncompress::Bunzip2->new( $self->archive ) or
+#                 return $self->_error(loc("Unable to open '%1': %2",
+#                             $self->archive,
+#                             $IO::Uncompress::Bunzip2::Bunzip2Error));
+# 
+#     my $fh = FileHandle->new('>'. $self->_gunzip_to) or
+#         return $self->_error(loc("Could not open '%1' for writing: %2",
+#                             $self->_gunzip_to, $! ));
+# 
+#     my $buffer;
+#     $fh->print($buffer) while $bz->read($buffer) > 0;
+#     $fh->close;
+# 
+#     ### set what files where extract, and where they went ###
+#     $self->files( [$self->_gunzip_to] );
+#     $self->extract_path( File::Spec->rel2abs(cwd()) );
+# 
+#     return 1;
+# }
+
+sub _bunzip2_cz2 {
+    my $self = shift;
+
+    my $use_list = { 'IO::Uncompress::Bunzip2' => '0.0' };
+    unless( can_load( modules => $use_list ) ) {
+        return $self->_error(loc("You do not have '%1' installed - Please " .
+                        "install it as soon as possible.",
+                        'IO::Uncompress::Bunzip2'));
+    }
+
+    IO::Uncompress::Bunzip2::bunzip2($self->archive => $self->_gunzip_to)
+        or return $self->_error(loc("Unable to uncompress '%1': %2",
+                            $self->archive,
+                            $IO::Uncompress::Bunzip2::Bunzip2Error));
+
+    ### set what files where extract, and where they went ###
+    $self->files( [$self->_gunzip_to] );
+    $self->extract_path( File::Spec->rel2abs(cwd()) );
+
+    return 1;
+}
 
 
 #################################
@@ -927,31 +1090,39 @@ sub _bunzip2_bin {
 #
 #################################
 
-### Error handling, the way Archive::Tar does it ###
-{
-    my $error       = '';
-    my $longmess    = '';
-
-    sub _error {
-        my $self    = shift;
-        $error      = shift;
-        $longmess   = Carp::longmess($error);
-
-        ### set Archive::Tar::WARN to 0 to disable printing
-        ### of errors
-        if( $WARN ) {
-            carp $DEBUG ? $longmess : $error;
-        }
-
-        return;
+sub _error {
+    my $self    = shift;
+    my $error   = shift;
+    
+    $self->_error_msg( $error );
+    $self->_error_msg_long( Carp::longmess($error) );
+    
+    ### set $Archive::Extract::WARN to 0 to disable printing
+    ### of errors
+    if( $WARN ) {
+        carp $DEBUG ? $self->_error_msg_long : $self->_error_msg;
     }
 
-    sub error {
-        my $self = shift;
-        return shift() ? $longmess : $error;
-    }
+    return;
 }
 
+sub error {
+    my $self = shift;
+    return shift() ? $self->_error_msg_long : $self->_error_msg;
+}
+
+sub _no_buffer_files {
+    my $self = shift;
+    my $file = shift or return;
+    return loc("No buffer captured, unable to tell ".
+               "extracted files or extraction dir for '%1'", $file);
+}
+
+sub _no_buffer_content {
+    my $self = shift;
+    my $file = shift or return;
+    return loc("No buffer captured, unable to get content for '%1'", $file);
+}
 1;
 
 =pod
@@ -978,11 +1149,16 @@ your archives do not have any of the extensions as described in the
 C<new()> method, you will have to specify the type explicitly, or
 C<Archive::Extract> will not be able to extract the archive for you.
 
-=head2 Bzip2 Support
+=head2 Supporting Very Large Files
 
-There's currently no very reliable pure perl Bzip2 implementation
-available, so C<Archive::Extract> can only extract C<bzip2> 
-compressed archives if you have a C</bin/bunzip2> program.
+C<Archive::Extract> can use either pure perl modules or command line
+programs under the hood. Some of the pure perl modules (like 
+C<Archive::Tar> take the entire contents of the archive into memory,
+which may not be feasible on your system. Consider setting the global
+variable C<$Archive::Extract::PREFER_BIN> to C<1>, which will prefer
+the use of command line programs and won't consume so much memory.
+
+See the C<GLOBAL VARIABLES> section below for details.
 
 =head1 GLOBAL VARIABLES
 
@@ -1026,20 +1202,20 @@ Defaults to C<false>.
 Maybe this module should use something like C<File::Type> to determine
 the type, rather than blindly trust the suffix.
 
-=head1 AUTHORS
+=back
 
-This module by
-Jos Boumans E<lt>kane@cpan.orgE<gt>.
+=head1 BUG REPORTS
+
+Please report bugs or other issues to E<lt>bug-archive-extract@rt.cpan.org<gt>.
+
+=head1 AUTHOR
+
+This module by Jos Boumans E<lt>kane@cpan.orgE<gt>.
 
 =head1 COPYRIGHT
 
-This module is
-copyright (c) 2004 Jos Boumans E<lt>kane@cpan.orgE<gt>.
-All rights reserved.
-
-This library is free software;
-you may redistribute and/or modify it under the same
-terms as Perl itself.
+This library is free software; you may redistribute and/or modify it 
+under the same terms as Perl itself.
 
 =cut
 
