@@ -13,7 +13,7 @@ BEGIN {
                         $USE_IPC_RUN $USE_IPC_OPEN3 $WARN
                     ];
 
-    $VERSION        = '0.32';
+    $VERSION        = '0.36';
     $VERBOSE        = 0;
     $DEBUG          = 0;
     $WARN           = 1;
@@ -24,6 +24,8 @@ BEGIN {
     @EXPORT_OK      = qw[can_run run];
 }
 
+require Carp;
+use File::Spec;
 use Params::Check               qw[check];
 use Module::Load::Conditional   qw[can_load];
 use Locale::Maketext::Simple    Style => 'gettext';
@@ -185,7 +187,10 @@ sub can_run {
         return MM->maybe_command($command);
 
     } else {
-        for my $dir (split /\Q$Config::Config{path_sep}\E/, $ENV{PATH}) {
+        for my $dir (
+            (split /\Q$Config::Config{path_sep}\E/, $ENV{PATH}),
+            File::Spec->curdir
+        ) {           
             my $abs = File::Spec->catfile($dir, $command);
             return $abs if $abs = MM->maybe_command($abs);
         }
@@ -295,7 +300,7 @@ sub run {
     };
 
     unless( check( $tmpl, \%hash, $VERBOSE ) ) {
-        carp(loc("Could not validate input: %1", Params::Check->last_error));
+        Carp::carp(loc("Could not validate input: %1", Params::Check->last_error));
         return;
     };        
 
@@ -583,10 +588,13 @@ sub _system_run {
 
         for my $name ( @fds ) {
             my($redir, $fh, $glob) = @{$Map{$name}} or (
-                carp(loc("No such FD: '%1'", $name)), next );
+                Carp::carp(loc("No such FD: '%1'", $name)), next );
             
-            open $glob, $redir,$fh or (
-                        carp(loc("Could not dup '$name': %1", $!)),
+            ### MUST use the 2-arg version of open for dup'ing for 
+            ### 5.6.x compatibilty. 5.8.x can use 3-arg open
+            ### see perldoc5.6.2 -f open for details            
+            open $glob, $redir . fileno($fh) or (
+                        Carp::carp(loc("Could not dup '$name': %1", $!)),
                         return
                     );        
                 
@@ -594,7 +602,7 @@ sub _system_run {
             ### just dup it
             if( $redir eq '>&' ) {
                 open( $fh, '>', File::Spec->devnull ) or (
-                    carp(loc("Could not reopen '$name': %1", $!)),
+                    Carp::carp(loc("Could not reopen '$name': %1", $!)),
                     return
                 );
             }
@@ -612,10 +620,13 @@ sub _system_run {
 
         for my $name ( @fds ) {
             my($redir, $fh, $glob) = @{$Map{$name}} or (
-                carp(loc("No such FD: '%1'", $name)), next );
+                Carp::carp(loc("No such FD: '%1'", $name)), next );
 
-            open( $fh, $redir,$glob ) or (
-                    carp(loc("Could not restore '$name': %1", $!)),
+            ### MUST use the 2-arg version of open for dup'ing for 
+            ### 5.6.x compatibilty. 5.8.x can use 3-arg open
+            ### see perldoc5.6.2 -f open for details
+            open( $fh, $redir . fileno($glob) ) or (
+                    Carp::carp(loc("Could not restore '$name': %1", $!)),
                     return
                 ); 
            
@@ -632,7 +643,6 @@ sub _debug {
     my $msg     = shift or return;
     my $level   = shift || 0;
     
-    require Carp;
     local $Carp::CarpLevel += $level;
     Carp::carp($msg);
     
@@ -663,7 +673,7 @@ settings honored cleanly.
 Otherwise, if the variable C<$IPC::Cmd::USE_IPC_OPEN3> is set to true 
 (See the C<GLOBAL VARIABLES> Section), try to execute the command using
 C<IPC::Open3>. Buffers will be available on all platforms except C<Win32>,
-interactive commands will still execute cleanly, and also your  verbosity
+interactive commands will still execute cleanly, and also your verbosity
 settings will be adhered to nicely;
 
 =item *
@@ -756,22 +766,22 @@ however, since you can just inspect your buffers for the contents.
 
 C<IPC::Run>, C<IPC::Open3>
 
-=head1 AUTHOR
-
-This module by
-Jos Boumans E<lt>kane@cpan.orgE<gt>.
-
 =head1 ACKNOWLEDGEMENTS
 
 Thanks to James Mastros and Martijn van der Streek for their
 help in getting IPC::Open3 to behave nicely.
 
+=head1 BUG REPORTS
+
+Please report bugs or other issues to E<lt>bug-ipc-cmd@rt.cpan.orgE<gt>.
+
+=head1 AUTHOR
+
+This module by Jos Boumans E<lt>kane@cpan.orgE<gt>.
+
 =head1 COPYRIGHT
 
-This module is
-copyright (c) 2002 - 2006 Jos Boumans E<lt>kane@cpan.orgE<gt>.
-All rights reserved.
+This library is free software; you may redistribute and/or modify it 
+under the same terms as Perl itself.
 
-This library is free software;
-you may redistribute and/or modify it under the same
-terms as Perl itself.
+=cut
