@@ -3,7 +3,7 @@ use strict;
 
 use locale;
 use vars qw ($VERSION @ISA @REGEXS);
-$VERSION = 0.76;
+$VERSION = 0.7203;
 
 push @REGEXS, qr/
 	^v?	# optional leading 'v'
@@ -21,17 +21,6 @@ use overload (
     'nomethod' => \&vnoop,
 );
 
-my $VERSION_MAX = 0x7FFFFFFF;
-
-eval "use warnings";
-if ($@) {
-    eval '
-	package warnings;
-	sub enabled {return $^W;}
-	1;
-    ';
-}
-
 sub new
 {
 	my ($class, $value) = @_;
@@ -84,7 +73,6 @@ sub new
 	my $alpha = 0;
 	my $width = 3;
 	my $saw_period = 0;
-	my $vinf = 0;
 	my ($start, $last, $pos, $s);
 	$s = 0;
 
@@ -103,8 +91,9 @@ sub new
 	while ( substr($value,$pos,1) =~ /[._\d]/ ) {
 	    if ( substr($value,$pos,1) eq '.' ) {
 		if ($alpha) {
+		    require Carp;
 		    Carp::croak("Invalid version format ".
-		      "(underscores before decimal)");
+		    	"(underscores before decimal)");
 		}
 		$saw_period++;
 		$last = $pos;
@@ -113,7 +102,7 @@ sub new
 		if ($alpha) {
 		    require Carp;
 		    Carp::croak("Invalid version format ".
-			"(multiple underscores)");
+		    	"(multiple underscores)");
 		}
 		$alpha = 1;
 		$width = $pos - $last - 1; # natural width of sub-version
@@ -123,21 +112,18 @@ sub new
 
 	if ( $alpha && !$saw_period ) {
 	    require Carp;
-	    Carp::croak("Invalid version format ".
-		"(alpha without decimal)");
+	    Carp::croak("Invalid version format (alpha without decimal)");
 	}
 
 	if ( $alpha && $saw_period && $width == 0 ) {
 	    require Carp;
-	    Carp::croak("Invalid version format ".
-		"(misplaced _ in number)");
+	    Carp::croak("Invalid version format (misplaced _ in number)");
 	}
 
 	if ( $saw_period > 1 ) {
 	    $qv = 1; # force quoted version processing
 	}
 
-	$last = $pos;
 	$pos = $s;
 
 	if ( $qv ) {
@@ -177,14 +163,9 @@ sub new
 			    $orev = $rev;
 			    $rev += substr($value,$s,1) * $mult;
 			    $mult /= 10;
-			    if (   abs($orev) > abs($rev) 
-				|| abs($rev) > abs($VERSION_MAX) ) {
-				if ( warnings::enabled("overflow") ) {
-				    require Carp;
-				    Carp::carp("Integer overflow in version");
-				}
-				$s = $end - 1;
-				$rev = $VERSION_MAX;
+			    if ( abs($orev) > abs($rev) ) {
+				require Carp;
+				Carp::croak("Integer overflow in version");
 			    }
 			    $s++;
 			    if ( substr($value,$s,1) eq '_' ) {
@@ -197,14 +178,9 @@ sub new
 			    $orev = $rev;
 			    $rev += substr($value,$end,1) * $mult;
 			    $mult *= 10;
-			    if (   abs($orev) > abs($rev) 
-				|| abs($rev) > abs($VERSION_MAX) ) {
-				if ( warnings::enabled("overflow") ) {
-				    require Carp;
-				    Carp::carp("Integer overflow in version");
-				}
-				$end = $s - 1;
-				$rev = $VERSION_MAX;
+			    if ( abs($orev) > abs($rev) ) {
+				require Carp;
+				Carp::croak("Integer overflow in version");
 			    }
 			}
 		    }
@@ -252,21 +228,12 @@ sub new
 	}
 
 	if ( substr($value,$pos) ) { # any remaining text
-	    if ( warnings::enabled("misc") ) {
-		require Carp;
-		Carp::carp("Version string '$value' contains invalid data; ".
-		     "ignoring: '".substr($value,$pos)."'");
-	    }
+	    warn "Version string '$value' contains invalid data; ".
+	         "ignoring: '".substr($value,$pos)."'";
 	}
 
 	# cache the original value for use when stringification
-	if ( $vinf ) {
-	    $self->{vinf} = 1;
-	    $self->{original} = 'v.Inf';
-	}
-	else {
-	    $self->{original} = substr($value,0,$pos);
-	}
+	$self->{original} = substr($value,0,$pos);
 
 	return ($self);
 }
@@ -355,11 +322,7 @@ sub stringify
 	require Carp;
 	Carp::croak("Invalid version object");
     }
-    return exists $self->{original} 
-    	? $self->{original} 
-	: exists $self->{qv} 
-	    ? $self->normal
-	    : $self->numify;
+    return $self->{original};
 }
 
 sub vcmp
@@ -490,8 +453,7 @@ sub _un_vstring {
 # Thanks to Yitzchak Scott-Thoennes for this mode of operation
 {
     local $^W;
-    *UNIVERSAL::VERSION # Module::Build::ModuleInfo doesn't see this now
-      = sub {
+    *UNIVERSAL::VERSION = sub {
 	my ($obj, $req) = @_;
 	my $class = ref($obj) || $obj;
 
