@@ -2,11 +2,10 @@ package CPANPLUS::Dist::Base;
 
 use strict;
 
-use base    qw[CPANPLUS::Dist];
-use vars    qw[$VERSION];
+use vars    qw[@ISA $VERSION];
+@ISA =      qw[CPANPLUS::Dist];
 $VERSION =  '0.01';
 
-
 =head1 NAME
 
 CPANPLUS::Dist::Base - Base class for custom distribution classes
@@ -57,16 +56,6 @@ class are called:
 
 =cut
 
-=head2 @subs = $Class->methods
-
-Returns a list of methods that this class implements that you can
-override.
-
-=cut
-
-sub methods { 
-    return qw[format_available init prepare create install uninstall] 
-}
 
 =head2 $bool = $Class->format_available
 
