@@ -142,7 +142,7 @@ sub format_available {
         return;
     }
     
-    for my $pgm ( qw[make perlwrapper] ) {
+    for my $pgm ( qw[make] ) {
         unless( $conf->get_program( $pgm ) ) { 
             error(loc(
                 "You do not have '%1' in your path -- '%2' not available\n" .
@@ -217,8 +217,7 @@ sub prepare {
         error( loc( "No dir found to operate on!" ) );
         return;
     }
- 
-$DB::single = 1; 
+    
     my $args;
     my( $force, $verbose, $perl, $mmflags );
     {   local $Params::Check::ALLOW_UNKNOWN = 1;
@@ -377,7 +376,7 @@ $DB::single = 1;
         
         ### start resolving prereqs ###
         my $prereqs = $self->status->prereqs;
-        
+       
         ### a hashref of prereqs on success, undef on failure ###
         $prereqs    ||= $dist->_find_prereqs( 
                                     verbose => $verbose,
@@ -654,7 +653,9 @@ sub create {
                 ### send success on force...
                 $test_fail++;
                 
-                unless( $force ) {
+                if( !$force and !$cb->_callbacks->proceed_on_test_failure->(
+                                      $self, $captured ) 
+                ) {
                     $fail++; last RUN;     
                 }
             }
