@@ -1,10 +1,11 @@
 package CPANPLUS::Internals::Utils;
 
 use strict;
-use CPANPLUS::inc;
+
 use CPANPLUS::Error;
 use CPANPLUS::Internals::Constants;
 
+use Cwd;
 use File::Copy;
 use Params::Check               qw[check];
 use Module::Load::Conditional   qw[can_load];
@@ -56,7 +57,9 @@ sub _mkdir {
         dir     => { required => 1 },
     };
 
-    my $args = check( $tmpl, \%hash ) or return;
+    my $args = check( $tmpl, \%hash ) or (
+        error(loc( Params::Check->last_error ) ), return
+    );       
 
     unless( can_load( modules => { 'File::Path' => 0.0 } ) ) {
         error( loc("Could not use File::Path! This module should be core!") );
@@ -368,6 +371,161 @@ sub _vcmp {
     return $x <=> $y;
 }
 
+=head2 $cb->_home_dir
+
+Returns the user's homedir, or C<cwd> if it could not be found
+
+=cut
+
+sub _home_dir {
+    my @os_home_envs = qw( APPDATA HOME USERPROFILE WINDIR SYS$LOGIN );
+
+    for my $env ( @os_home_envs ) {
+        next unless exists $ENV{ $env };
+        next unless defined $ENV{ $env } && length $ENV{ $env };
+        return $ENV{ $env } if -d $ENV{ $env };
+    }
+
+    return cwd();
+}
+
+=head2 $path = $cb->_safe_path( path => $path );
+
+Returns a path that's safe to us on Win32. Only cleans up
+the path on Win32 if the path exists.
+
+=cut
+
+sub _safe_path {
+    my $self = shift;
+    
+    my %hash = @_;
+    
+    my $path;
+    my $tmpl = {
+        path  => { required => 1,     store => \$path },
+    };       
+
+    check( $tmpl, \%hash ) or return;
+    
+    ### only need to fix it up if there's spaces in the path   
+    return $path unless $path =~ /\s+/;
+    
+    ### or if we are on win32
+    return $path if $^O ne 'MSWin32';
+
+    ### clean up paths if we are on win32
+    return Win32::GetShortPathName( $path ) || $path;
+
+}
+
+
+=head2 ($pkg, $version, $ext) = $cb->_split_package_string( package => PACKAGE_STRING );
+
+Splits the name of a CPAN package string up in it's package, version 
+and extension parts.
+
+For example, C<Foo-Bar-1.2.tar.gz> would return the following parts:
+
+    Package:    Foo-Bar
+    Version:    1.2
+    Extension:  tar.gz
+
+=cut
+
+{   my $del_re = qr/[-_\+]/i;           # delimiter between elements
+    my $pkg_re = qr/[a-z]               # any letters followed by 
+                    [a-z\d]*            # any letters, numbers
+                    (?i:\.pm)?          # followed by '.pm'--authors do this :(
+                    (?:                 # optionally repeating:
+                        $del_re         #   followed by a delimiter
+                        [a-z]           #   any letters followed by 
+                        [a-z\d]*        #   any letters, numbers                        
+                        (?i:\.pm)?      # followed by '.pm'--authors do this :(
+                    )*
+                /xi;   
+    
+    my $ver_re = qr/[a-z]*\d+[a-z]*     # contains a digit and possibly letters
+                    (?:
+                        [-._]           # followed by a delimiter
+                        [a-z\d]+        # and more digits and or letters
+                    )*?
+                /xi;
+ 
+    my $ext_re = qr/[a-z]               # a letter, followed by
+                    [a-z\d]*            # letters and or digits, optionally
+                    (?:                 
+                        \.              #   followed by a dot and letters
+                        [a-z\d]+        #   and or digits (like .tar.bz2)
+                    )?                  #   optionally
+                /xi;
+
+    my $ver_ext_re = qr/
+                        ($ver_re+)      # version, optional
+                        (?:
+                            \.          # a literal .
+                            ($ext_re)   # extension,
+                        )?              # optional, but requires version
+                /xi;
+                
+    ### composed regex for CPAN packages
+    my $full_re = qr/
+                    ^
+                    ($pkg_re+)          # package
+                    (?: 
+                        $del_re         # delimiter
+                        $ver_ext_re     # version + extension
+                    )?
+                    $                    
+                /xi;
+                
+    ### composed regex for perl packages
+    my $perl    = PERL_CORE;
+    my $perl_re = qr/
+                    ^
+                    ($perl)             # package name for 'perl'
+                    (?:
+                        $ver_ext_re     # version + extension
+                    )?
+                    $
+                /xi;       
+
+
+sub _split_package_string {
+        my $self = shift;
+        my %hash = @_;
+        
+        my $str;
+        my $tmpl = { package => { required => 1, store => \$str } };
+        check( $tmpl, \%hash ) or return;
+        
+        
+        ### 2 different regexes, one for the 'perl' package, 
+        ### one for ordinary CPAN packages.. try them both, 
+        ### first match wins.
+        for my $re ( $full_re, $perl_re ) {
+            
+            ### try the next if the match fails
+            $str =~ $re or next;
+
+            my $pkg = $1 || ''; 
+            my $ver = $2 || '';
+            my $ext = $3 || '';
+
+            ### this regex resets the capture markers!
+            ### strip the trailing delimiter
+            $pkg =~ s/$del_re$//;
+            
+            ### strip the .pm package suffix some authors insist on adding
+            $pkg =~ s/\.pm$//i;
+
+            return ($pkg, $ver, $ext );
+        }
+        
+        return;
+    }
+}
+
 1;
 
 # Local variables:
