@@ -72,6 +72,7 @@ my $tmpl = {
     _status     => { no_override => 1 },                # stores status object
     author      => { default => '', required => 1,
                      allow => IS_AUTHOBJ },             # module author
+    mtime       => { default => '' },
 };
 
 ### autogenerate accessors ###
@@ -93,7 +94,8 @@ Returns a list of all accessor methods to the object
 
 =cut
 
-sub accessors { return keys %$tmpl };
+### *name is an alias, include it explicitly
+sub accessors { return ('name', keys %$tmpl) };
 
 =head1 ACCESSORS
 
@@ -359,21 +361,28 @@ L<Module::ThirdParty> for more details.
 
 =cut
 
-{
-    my $regex = qr/^(.+)-(.+)\.((?:tar\.gz|zip|tgz))/i;
-
-    ### fetches the test reports for a certain module ###
-    sub package_name {
-        return $1 if shift->package() =~ $regex;
-    }
-
-    sub package_version {
-        return $2 if shift->package() =~ $regex;
-    }
-
-    sub package_extension {
-        return $3 if shift->package() =~ $regex;
-    }
+{   ### fetches the test reports for a certain module ###
+    my %map = (
+        name        => 0,
+        version     => 1,
+        extension   => 2,
+    );        
+    
+    while ( my($type, $index) = each %map ) {
+        my $name    = 'package_' . $type;
+        
+        no strict 'refs';
+        *$name = sub {
+            my $self = shift;
+            my @res  = $self->parent->_split_package_string(     
+                            package => $self->package 
+                       );
+     
+            ### return the corresponding index from the result
+            return $res[$index] if @res;
+            return;
+        };
+    }        
 
     sub package_is_perl_core {
         my $self = shift;
@@ -387,8 +396,9 @@ L<Module::ThirdParty> for more details.
             ### if the package is newer than installed, then it's dual-lifed
             return if $self->version > $self->installed_version;
 
-            ### if the package is newer than corelist, then it's dual-lifed
-            return if $self->version > $core;
+            ### if the package is newer or equal to the corelist, 
+            ### then it's dual-lifed
+            return if $self->version >= $core;
 
             ### otherwise, it's older than corelist, thus unsuitable.
             return 1;
@@ -744,6 +754,7 @@ sub install {
         $args = check( $tmpl, \%hash ) or return;
     }
 
+
     ### if this target isn't 'install', we will need to at least 'create' 
     ### every prereq, so it can build
     ### XXX prereq_target of 'prepare' will do weird things here, and is
@@ -1103,9 +1114,11 @@ this:
     Author                  Jarkko Hietaniemi (jhi@iki.fi)
     Description             High resolution time, sleep, and alarm
     Development Stage       Released
+    Installed File          /usr/local/perl/lib/Time/Hires.pm
     Interface Style         plain Functions, no references used
     Language Used           C and perl, a C compiler will be needed
     Package                 Time-HiRes-1.65.tar.gz
+    Public License          Unknown
     Support Level           Developer
     Version Installed       1.52
     Version on CPAN         1.65
@@ -1130,6 +1143,7 @@ sub details {
     ### if so, add version have and version on cpan
     $res->{'Version Installed'} = $self->installed_version
                                     if $self->installed_version;
+    $res->{'Installed File'} = $self->installed_file if $self->installed_file;
 
     my $i = 0;
     for my $item( split '', $self->dslip ) {
@@ -1531,6 +1545,25 @@ sub best_path_to_module_build {
     return;
 }
 
+=pod
+
+=head1 BUG REPORTS
+
+Please report bugs or other issues to E<lt>bug-cpanplus@rt.cpan.org<gt>.
+
+=head1 AUTHOR
+
+This module by Jos Boumans E<lt>kane@cpan.orgE<gt>.
+
+=head1 COPYRIGHT
+
+The CPAN++ interface (of which this module is a part of) is copyright (c) 
+2001 - 2007, Jos Boumans E<lt>kane@cpan.orgE<gt>. All rights reserved.
+
+This library is free software; you may redistribute and/or modify it 
+under the same terms as Perl itself.
+
+=cut
 
 # Local variables:
 # c-indentation-style: bsd
